﻿# -*- coding: utf-8 -*-
"""
Inserts an error list in HTML.
"""

import cse
import itertools

# Settings for the HTML source.
htmlStyleNode = '''<style type="text/css">
	table.errorlistout	{ margin-bottom: 25pt; }
	td.errorlistout		{ border-width: 0; padding: 0; }
	div.errorlistin		{ max-height: 250px; overflow-y: scroll; margin: 0; }
	table.errorlistin	{ width: 100%; border-collapse: collapse; border-width: 0pt; table-layout: fixed; }
	table.errorlistin td
		{ border-width: 1pt 1pt 0 0; vertical-align: top; }
	table.errorlistin tr:first-child td
		{ border-top-width: 0pt; }
	table.errorlistin td.errorsdetail
		{ width: 60%; white-space: nowrap; overflow: hidden; text-overflow: ellipsis; }
	table.errorlistin td.errorsdetail:hover
		{  white-space: normal; }
	table.errorlistin td.errorsdetail:hover a
		{  display: block; }
</style>'''

htmlTableWrap = '<table class="out errorlistout NOPRINT"><tbody><tr><td class="errorlistout"><div class="errorlistin"><table class="errorlistin"><tbody /></table></div></td></tr></tbody></table>'


# Method for checking output sheets.
def InsertErrorList(s, e):
  # Plugin execution event data.
  v = e.Value
  
  # Loop for each SheetTabInfo.
  for sti in v.sheets:
    # Search for HasError replacement node.
    doc = sti.DocumentTemplateParent
    root = doc.DocumentNode
    cmmErrSpecNode = root.SelectSingleNode("//comment()[contains(.,'{\"spec\":\"HasError\"}') or contains(.,\"{'spec':'HasError'}\")]")
    
    # Ignore if no HasError replacement node is found.
    if cmmErrSpecNode is None:
      continue
    
    # Ignore if there is no error information.
    if len(sti.Sheet.ExtInfo.Errors) == 0:
      continue
    
    # Add style mode.
    nStyle = root.CreateNode(htmlStyleNode)
    nCurHead = root.SelectSingleNode("//head")
    if nCurHead is not None:
      # Place at the end of the head element.
      nCurHead.AppendChild(doc.CreateTextNode("\n"))
      nCurHead.AppendChild(nStyle)
      nCurHead.AppendChild(doc.CreateTextNode("\n"))
    else:
      # If there is no head, add to the end of the root node.
      root.AppendChild(doc.CreateTextNode("\n"))
      root.AppendChild(nStyle)
      root.AppendChild(doc.CreateTextNode("\n"))
    
    
    # Add a table node to the end of the HasError replacement node.
    nReturn = doc.CreateTextNode("\n")
    nTableWrap = root.CreateNode(htmlTableWrap)
    cmmErrSpecNode.ParentNode.InsertAfter(nReturn, cmmErrSpecNode)
    cmmErrSpecNode.ParentNode.InsertAfter(nTableWrap, nReturn)
    nTableBody = nTableWrap.SelectSingleNode("tbody/tr/td/div/table/tbody")
    
    # Add a line to each error object list.
    for (key, Title), g in itertools.groupby(sti.Sheet.ExtInfo.Errors, lambda x: (x.ErrTitleKey, x.ErrTitleString)):
      # Groups using the ErrorItem.ErrTitleKey and ErrTitleString tuples.
      nFirstTd = None	# The item name Td. 
      lastIndex = 0
      
      for i, erritem in enumerate(g):
        lastIndex = i
        nTr = None
        elmLink = None
        if i == 0:
          # First row in the grouped set.
          nTr = root.CreateNode('<tr><td class="errorstitle" /><td class="errorsdetail"><a /></td></tr>')
          nFirstTd = nTr.SelectSingleNode("td[1]")
          nFirstTd.AppendChild(doc.CreateTextNode(erritem.ErrTitleString))
          elmLink = nTr.SelectSingleNode("td[2]/a")
          
        else:
          # Second and subsequent rows in the grouped set.
          nTr = root.CreateNode('<tr><td class="errorsdetail"><a /></td></tr>')
          elmLink = nTr.SelectSingleNode("td[1]/a")
        
        # Adds links.
        elmLink.AppendChild(doc.CreateTextNode(erritem.ErrMsgString))
        elmLink.SetAttributeValue('title', erritem.ErrMsgString)	# For tool tip.
        if erritem.Path is not None:
          elmLink.SetAttributeValue('href', '#' + str(erritem.Path).replace('[', '').replace(']', '') + '.value')
        
        # Add a column to the table.
        nTableBody.AppendChild(nTr)
      
      # The rowspan setting.
      nFirstTd.SetAttributeValue('rowspan', str(lastIndex + 1))
    


cse.addhandler("BeforeMakeCheckSheet", InsertErrorList)
